<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Flames Example</H3>
            <p>Author: Pete Shinners (pete at shinners.org)<br>
            Submission date: April 03, 2001</p>
	  
            <p>Description: Example of a realtime fire effect. Makes heavy use of Numeric python and the surfarray module in pygame.</p>
            <p align="center"><img src="flames.jpg" width="240" height="180"></p>

            <p>Download: <a href="flames.zip">flames.zip</a></p>	  

            <p>pygame version required: Any (Numeric required)<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: This submission reminds us of a few important optimization techniques. First, use the right tool for the job. Pete is doing a recursive blur, a per-pixel activity. The surfarray extensions to pygame are almost mandatory here - you could do it with get_at() and set_at(), but the cost in speed would be prohibitive. Second, do your work in integers where possible. Python doesn't free you from the costs associated with floating-point mathematics. Finally, it's often possible to create an effect and then scale it up for display. The results may not be as precise as a full-sized effect, but the speed will be much greater, and for some effects (blur comes to mind), the difference won't be visible.</p>

            <p>The code has been updated - it is optimized further, and the new speed improvements have allowed the resolution to be increased.</p>

            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python

"""flames.py - Realtime Fire Effect Demo
Pete Shinners, April 3, 2001

On December 6, 60% speedup by using the pygame.transform.scale
function instead of doing a simple Numeric scale.

Ok, this is a pretty intense demonstation of using
the surfarray module and numeric. It uses an 8bit
surfaces with a colormap to represent the fire. We
then go crazy on a separate Numeric array to get
realtime fire. I'll try to explain my methods here...

This flame algorithm is very popular, and you can find
it for just about all graphic libraries. The fire effect
works by placing random values on the bottom row of the
image. Then doing a simplish blur that is weighted to
move the values upward. Then slightly darken the image
so the colors get darker as they move up. The secret is
that the blur routine is "recursive" so when you blur
the 2nd row, the values there are used when you blur
the 3rd row, and all the way up.

This fire algorithm works great, but the bottom rows
are usually a bit ugly. In this demo we just render
a fire surface that has 3 extra rows at the bottom we
just don't use.

Also, the fire is rendered at half the resolution of
the full image. We then simply double the size of the
fire data before applying to the surface.

Several of these techniques are covered in the pygame
surfarray tutorial. doubling an image, and the fancy
blur is just a modified version of what is in the tutorial.

This runs at about 40fps on my celeron-400
"""</span>


import pygame, pygame.transform
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *
from RandomArray import *

RES = array((280, 200))
MAX = 246
RESIDUAL = 86
HSPREAD, VSPREAD = 26, 78
VARMIN, VARMAX = -2, 3

<span class="pydef">def main():</span>
    <span class="pycomment">"main function called when the script is run"
    #first we just init pygame and create some empty arrays to work with </span>   
    pygame.init()
    screen = pygame.display.set_mode(RES, 0, 8)
    setpalette(screen)
    flame = zeros(RES/2 + (0,3))
    miniflame = pygame.Surface((RES[0]/2, RES[1]/2), 0, 8)
    miniflame.set_palette(screen.get_palette())
    randomflamebase(flame)    

    while 1:
        for e in pygame.event.get():
            if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                return
            
        modifyflamebase(flame)
        processflame(flame)
        blitdouble(screen, flame, miniflame)
        pygame.display.flip()



<span class="pydef">def setpalette(screen):</span>
    <span class="pycomment">"here we create a numeric array for the colormap"</span>
    gstep, bstep = 75, 150
    cmap = zeros((256, 3))
    cmap[:,0] = minimum(arange(256)*3, 255)
    cmap[gstep:,1] = cmap[:-gstep,0]
    cmap[bstep:,2] = cmap[:-bstep,0]
    screen.set_palette(cmap)


<span class="pydef">def randomflamebase(flame):</span>
    <span class="pycomment">"just set random values on the bottom row"</span>
    flame[:,-1] = randint(0, MAX, flame.shape[0])


<span class="pydef">def modifyflamebase(flame):</span>
    <span class="pycomment">"slightly change the bottom row with random values"</span>
    bottom = flame[:,-1]
    mod = randint(VARMIN, VARMAX, bottom.shape[0])
    add(bottom, mod, bottom)
    maximum(bottom, 0, bottom)
    <span class="pycomment">#if values overflow, reset them to 0</span>
    bottom[:] = choose(greater(bottom,MAX), (bottom,0))


<span class="pydef">def processflame(flame):</span>
    <span class="pycomment">"this function does the real work, tough to follow"</span>
    notbottom = flame[:,:-1]    

    <span class="pycomment">#first we multiply by about 60%</span>
    multiply(notbottom, 146, notbottom)
    right_shift(notbottom, 8, notbottom)

    <span class="pycomment">#work with flipped image so math accumulates.. magic!</span>
    flipped = flame[:,::-1]

    <span class="pycomment">#all integer based blur, pulls image up too</span>
    tmp = flipped * 20
    right_shift(tmp, 8, tmp)
    tmp2 = tmp >> 1
    add(flipped[1:,:], tmp2[:-1,:], flipped[1:,:])
    add(flipped[:-1,:], tmp2[1:,:], flipped[:-1,:])
    add(flipped[1:,1:], tmp[:-1,:-1], flipped[1:,1:])
    add(flipped[:-1,1:], tmp[1:,:-1], flipped[:-1,1:])

    tmp = flipped * 80
    right_shift(tmp, 8, tmp)
    add(flipped[:,1:], tmp[:,:-1]>>1, flipped[:,1:])
    add(flipped[:,2:], tmp[:,:-2], flipped[:,2:])

    <span class="pycomment">#make sure no values got too hot</span>
    minimum(notbottom, MAX, notbottom)


<span class="pydef">def blitdouble(screen, flame, miniflame):</span>
    <span class="pycomment">"double the size of the data, and blit to screen"</span>
    blit_array(miniflame, flame[:,:-3])
    s2 = pygame.transform.scale(miniflame, screen.get_size())
    screen.blit(s2, (0,0))


if __name__ == '__main__': main()


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

